module jp.ku6.tetris.MicrosoftWindowsBitmapImageFile;

import std.c.stdlib;
import std.c.stdio;
import jp.ku6.windows.game.graphics;

/++
 + ビットマップファイル
 +/
class MicrosoftWindowsBitmapImageFile : SurfaceSource
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  filePath = ファイルパス
	 +  offset = ファイル先頭からのオフセット。1ファイルに複数のファイルをまとめた時に使用
	 +/
	this(string filePath, uint offset)
	{
		filePath_ = filePath ~ 0;
		fileOffset_ = offset;
	}

	/++
	 + デストラクタ
	 +/
	~this()
	{
		close();
	}

	@property
	{
		uint width()
		{
			return width_;
		}

		uint height()
		{
			return height_;
		}

		string filePath()
		{
			return filePath_[0 .. $ - 1];
		}
	}

	bool open()
	{
		// ファイル自体をオープン
		file_ = std.c.stdio.fopen(filePath_.ptr, "rb\0".ptr);
		if (file_ is null)
			return false;

		// アーカイブ内のファイル先頭位置へ移動
		if (fileOffset_ > 0)
			std.c.stdio.fseek(file_, fileOffset_, std.c.stdio.SEEK_SET);

		// 必要なところだけ読み出し
		ushort[15] data;
		std.c.stdio.fread(data.ptr, ushort.sizeof, data.length, file_);

		width_ = cast(uint)data[9];
		height_ = cast(uint)data[11];

		// TODO: BPPをチェックする処理

		return true;
	}

	void close()
	{
		std.c.stdio.fclose(file_);

		width_ = 0;
		height_ = 0;
	}

	void read(uint* buffer, uint width, uint height)
	{
		// TODO: 24bit以外の時の処理

		// 1行のピクセルサイズを取得
		uint pitch = (width_ * 24 / 8 + 3) & (~3);

		// ピクセルデータ用のメモリを確保
		ubyte* source = cast(ubyte*)std.c.stdlib.malloc(pitch * height_);
		scope(exit) std.c.stdlib.free(source);

		// ピクセルデータ読み出し
		std.c.stdio.fseek(file_, fileOffset_ + 54, std.c.stdio.SEEK_SET);
		std.c.stdio.fread(source, ubyte.sizeof, pitch * height_, file_);

		for (short y; y < height_; ++y)
		{
			uint* d = &buffer[y * width];
			ubyte* s = &source[(height_ - y - 1) * pitch];

			for (short x; x < width_; ++x)
			{
				uint color = 0xff000000 | (s[x * 3 + 2] << 16) | (s[x * 3 + 1] << 8) | s[x * 3];
				if (color == TRANSFER_COLOR)
					d[x] = 0;
				else
					d[x] = color;
			}
		}
	}

private:

	// 抜き色
	immutable uint TRANSFER_COLOR = 0xff00ff00;

	// 幅
	uint width_;

	// 高さ
	uint height_;

	// アーカイブ中での個別ファイルの位置
	uint fileOffset_;

	// ファイルポインタ
	FILE* file_;

	// ファイルパス
	string filePath_;
}